/*
** Module   : ScrollableListImpl.java
** Abstract :
**
** Copyright (c) 2011-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SIY 20111127 Extracted from ScrollableList
** 002 SIY 20111221 Fixed constructor visibility.
** 003 GES 20140627 Changed color attribute method names to match naming standards.
** 004 CA  20140805 Do not allow ChUI impl classes access to APIs outside of ChuiOutputManager.
** 005 HC  20140911 GUI coordinate support - data types refactoring.
** 006 EVL 20150517 Adding api to set up the top row index.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.util.*;

/**
 * CHUI-specific portion of ScrollableList.
 */
public class ScrollableListImpl
extends ScrollableList<ChuiOutputManager>
{
   /**
    * Create component using given configuration and data model.
    * 
    * @param    box
    *           The instance with which this object must be associated.
    * @param    model
    *           The data model that represents the contents of the list.
    */
   ScrollableListImpl(ComboBox<ChuiOutputManager> box, ListModel<String> model)
   {
      super(box, model);
   }

   /**
    * Request focus.
    */
   @Override
   public void requestFocus()
   {
      super.requestFocus();

      screen().at(screenLocation().translate(0, currentRow()));
   }

   /**
    * Draws this component. Overrides draw() in JList.
    */
   @Override
   public void draw()
   {
      // get the absolute origin of this component
      Point origin = screenLocation();
      Color dc  = box.getDColor();
      
      String blanks = StringHelper.repeatChar(' ', (int) width());
      
      // in chui char coordinates are always whole numbers
      int y = (int) location().y;
      
      for (int i = 0; i < model().size() &&
                      y < visibleRows(); i++, y++, origin.y++)
      {
         if (y < 0)
            continue;
         
         screen().at(origin);

         Color color = (i == currentRow()) ? dc.withReverse() : dc;

         StringBuilder buf = new StringBuilder(getItemText(i));
         
         buf.append(blanks).setLength((int) width());

         screen().append(buf.toString(), color);
      }

      for (; y < visibleRows(); y++, origin.y++)
      {
         Color color = (y == 0 && currentRow() == -1) ? dc.withReverse() : dc;

         screen().at(origin);
         screen().append(blanks, color);
      }

      if (hasFocus() && isEnabled())
         drawCaret();
   }

   /**
    * Sets position of terminal's cursor to the first char of the combo-box.
    */
   @Override
   public void drawCaret()
   {
      // this check is required since the method can be called from multiple
      // drivers
      if (!hasFocus() || !isEnabled())
      {
         return;
      }

      Point origin = screenLocation();

      if (currentRow() > 0)
      {
          origin.y += currentRow();
      }

      screen().at(origin);
      requestSync();
   }

   /**
    * Set item number to be displayed on the top of the drop-down list.
    * 
    * @param   index
    *          The index of the top item.
    */
   @Override
   protected void setTopRow(int index)
   {
      // in ChUI just ensure the item is visible for now
      ensureRowVisible(index);
   }
}
